@@ -14,9 +14,10 @@
 #include <strstream>
 #include <fstream>
 #include <exception>
-#include <cstdio>
-#include <cstdlib>
-#include <ctime>
+#include <stdio.h>
+#include <stdlib.h>
+#include <time.h>
+#include <math.h>
 
 #include "Metric.h"
 #include "CUBException.h"
@@ -256,14 +257,14 @@ Region* Cube::create_region(int id, string name, long beginln, long endln,
 }
 
 int Cube::def_csite(int modid, int line, int callee_id) {
-  Module* mod;
+  // Module* mod;
   Region *caller, *callee;
   Callsite* csite;
   int id;
   
   /* new a call site object */
   try {
-    mod = (Module*) get_mod(modid);
+    // mod = (Module*) get_mod(modid);
     /* caller will be defined in the def_cnode() */
     // caller = find_caller_region(mod, line);
     caller = NULL;
@@ -282,13 +283,13 @@ int Cube::def_csite(int modid, int line, int callee_id) {
 
 Callsite* Cube::create_csite(int id, int modid, int line, 
 			     int callee_id, int caller_id) {
-  Module* mod;
+  // Module* mod;
   Region *caller, *callee;
   Callsite* csite;
   
   /* new a call site object */
   try {
-    mod = (Module*) get_mod(modid);
+    // mod = (Module*) get_mod(modid);
     caller = (Region*) get_region(caller_id);
     try {
       callee = (Region* )get_region(callee_id);
@@ -564,7 +565,7 @@ void Cube::validate_id(int id, int domaintype) {
 
   severity matrix, define values 
 ************************************************************************/  
-void Cube::set_sev(int met_id, int cnode_id, int thrd_id, const double value) {
+void Cube::set_sev(int met_id, int cnode_id, int thrd_id, double value) {
   severity[met_id][cnode_id][thrd_id] = value;
 }
 
@@ -886,7 +887,7 @@ double Cube::sev4view(double sev, int view, double refval, int pid) {
   }
 }
 
-int get_exponent(double max) {
+/*int get_exponent(double max) {
   unsigned long long val = (unsigned long long) max;
   int k = 0;
   while (val > 0) {
@@ -894,6 +895,31 @@ int get_exponent(double max) {
     k++;
   }
   return k;
+  }*/
+
+// for max >= 1
+int get_exponent(double max) {
+  if (max < 0) max = -max;
+  double val = floor(max);
+  int k = 0;
+  while (val > 0) {
+    val /= 10;
+    val = floor(val); // only integer part wanted
+    k++;
+  }
+  return k;
+}
+
+// for 0<max<1
+int get_exponent2(double max) {
+  if (max < 0) max = -max;
+  double val = max;
+  int k = 0;
+  while (val != 0 && val < 1) {
+    val *= 10;
+    k++;
+  }
+  return k;
 }
 
 /****************************************************************         
@@ -936,17 +962,28 @@ string Cube::gen_met_label(int met_id, double sev,
     disp_v = sev4view(sev, view, refval, met_id);
   }
   int exponent = get_exponent(max_sev);
-  // cout << "max sev:" << max_sev << "exponent: " << exponent << endl;
+
   if (uom.compare("occ") == 0) isInteger = true;
   if (view == 0) {// only if absolute view
-    if (!isInteger && max_sev >= 100.0) {
-      out.precision(2); // a.bc E xx
-      //out.setf(ios::scientific);
-      out.setf(ios::fixed); // omit the E xx part 
-      for (int m = 0; m < exponent-1; m++) sev /= 10.0;
-      out << sev << " " << p->get_name() << " ";
-      *(out.str()+out.pcount()) = 0;
-      return out.str();
+    if (!isInteger) {
+      
+      if (max_sev >= 100.0) {
+	out.precision(2); // a.bc E xx
+	out.setf(ios::fixed); // omit the E xx part 
+	for (int m = 0; m < exponent-1; m++) sev /= 10.0;
+	out << sev << " " << p->get_name() << " ";
+	*(out.str()+out.pcount()) = 0;
+	return out.str();
+      } else if (max_sev < 0.1) {
+	exponent = get_exponent2(max_sev);
+	//cout << "max sev:" << max_sev << "exponent: " << exponent << endl;
+	out.precision(2); // a.bc E xx
+	out.setf(ios::fixed); // omit the E xx part 
+	for (int m = 0; m < exponent; m++) sev *= 10.0;
+	out << sev << " " << p->get_name() << " ";
+	*(out.str()+out.pcount()) = 0;
+	return out.str();
+      }
     }
     if (isInteger && max_sev >= 1000) {
       out.precision(2); // a.bc (E xx, implied in spectrum)
@@ -972,18 +1009,20 @@ string Cube::gen_met_label(int met_id, double sev,
 
 string Cube::gen_max_label(double max_sev) {
   Metric* p = (Metric*) get_met(cur_metid);
-  double disp_v;
+  // double disp_v;
   bool isInteger = false;
   string uom = p->get_uom();
   ostrstream out;
 
   if (uom.compare("occ") == 0) isInteger = true;
-  if (!isInteger && max_sev >= 100.0) {
-    out.precision(2); // a.bc E xx
-    out.setf(ios::scientific);
-    out << max_sev << " ";
-    *(out.str()+out.pcount()) = 0;
-    return out.str();
+  if (!isInteger) {
+    if (max_sev >= 100.0 || max_sev < 0.1) {
+      out.precision(2); // a.bc E xx
+      out.setf(ios::scientific);
+      out << max_sev << " ";
+      *(out.str()+out.pcount()) = 0;
+      return out.str();
+    } 
   }
   if (isInteger && max_sev >= 1000) {
     out.precision(2); // a.bc E xx
@@ -1004,6 +1043,42 @@ string Cube::gen_max_label(double max_sev) {
   return out.str();
 }
 
+string Cube::gen_half_max_label(double max_sev) {
+  Metric* p = (Metric*) get_met(cur_metid);
+  // double disp_v;
+  bool isInteger = false;
+  string uom = p->get_uom();
+  ostrstream out;
+
+  if (uom.compare("occ") == 0) isInteger = true;
+  if (!isInteger) {
+    if (max_sev >= 100.0 || max_sev < 0.1) {
+      out.precision(2); // a.bc E xx
+      out.setf(ios::scientific);
+      out << max_sev/2 << " ";
+      *(out.str()+out.pcount()) = 0;
+      return out.str();
+    } 
+  }
+  if (isInteger && max_sev >= 1000) {
+    out.precision(2); // a.bc E xx
+    out.setf(ios::scientific);
+    out << max_sev/2 << " ";
+    *(out.str()+out.pcount()) = 0;
+    return out.str();    
+  } else if (isInteger) {
+    int z = (int) max_sev/2;
+    out << z << " ";
+    *(out.str()+out.pcount()) = 0;
+    return out.str();  
+  }
+  out.precision(2);
+  out.setf(ios::fixed);
+  out << max_sev/2 << " ";
+  *(out.str()+out.pcount()) = 0;
+  return out.str();
+}
+
 string Cube::gen_cnode_label(int met_id, int cnode_id, double sev, 
 			     int view, double refval, bool default_root) {
   Metric* p = (Metric*) get_met(met_id);
@@ -1026,18 +1101,27 @@ string Cube::gen_cnode_label(int met_id, int cnode_id, double sev,
   if (uom.compare("occ") == 0) isInteger = true;
   string name = ((Region*)cnode->get_csite()->get_callee())->get_name();
   if (view == 0) {// only if absolute view
-    if (!isInteger && max_sev >= 100.0) {
-      out.precision(2); // a.bc E xx
-      //out.setf(ios::scientific);
-      out.setf(ios::fixed);
-      for (int m = 0; m < exponent-1; m++) sev /= 10.0;
-      out << sev << " " << name << " ";
-      *(out.str()+out.pcount()) = 0;
-      return out.str();
+    if (!isInteger) {
+      if (max_sev >= 100.0) {
+	out.precision(2); // a.bc E xx
+	out.setf(ios::fixed);
+	for (int m = 0; m < exponent-1; m++) sev /= 10.0;
+	out << sev << " " << name << " ";
+	*(out.str()+out.pcount()) = 0;
+	return out.str();
+      } else if (max_sev < 0.1) {
+	out.precision(2); // a.bc E xx
+	out.setf(ios::fixed); // omit the E xx part 
+	exponent = get_exponent2(max_sev);
+	for (int m = 0; m < exponent; m++) sev *= 10.0;
+	out << sev << " " << p->get_name() << " ";
+	*(out.str()+out.pcount()) = 0;
+	return out.str();
+      }
+
     }
     if (isInteger && max_sev >= 1000) {
       out.precision(2); // a.bc E xx
-      //out.setf(ios::scientific);
       out.setf(ios::fixed);
       for (int m = 0; m < exponent-1; m++) sev /= 10.0;
       out << sev << " " << name << " ";
@@ -1081,14 +1165,24 @@ string Cube::gen_function_label(int met_id, int func_id, double sev,
   if (uom.compare("occ") == 0) isInteger = true;
   string name = r->get_name();
   if (view == 0) {// only if absolute view
-    if (!isInteger && max_sev >= 100.0) {
-      out.precision(2); // a.bc E xx
-      //out.setf(ios::scientific);
-      out.setf(ios::fixed);
-      for (int m = 0; m < exponent-1; m++) sev /= 10.0;
-      out << sev << " " << name << " ";
-      *(out.str()+out.pcount()) = 0;
-      return out.str();
+    if (!isInteger) {
+      if (max_sev >= 100.0) {
+	out.precision(2); // a.bc E xx
+	//out.setf(ios::scientific);
+	out.setf(ios::fixed);
+	for (int m = 0; m < exponent-1; m++) sev /= 10.0;
+	out << sev << " " << name << " ";
+	*(out.str()+out.pcount()) = 0;
+	return out.str();
+      } else if (max_sev < 0.1) {
+	out.precision(2); // a.bc E xx
+	out.setf(ios::fixed); // omit the E xx part 
+	exponent = get_exponent2(max_sev);
+	for (int m = 0; m < exponent; m++) sev *= 10.0;
+	out << sev << " " << p->get_name() << " ";
+	*(out.str()+out.pcount()) = 0;
+	return out.str();
+      } 
     }
     if (isInteger && max_sev >= 1000) {
       out.clear();
@@ -1138,14 +1232,24 @@ string Cube::gen_mod_label(int met_id, int mod_id, double sev,
   if (uom.compare("occ") == 0) isInteger = true;
   string name = mod->get_name();
   if (view == 0) {// only if absolute view
-    if (!isInteger && max_sev >= 100.0) {
-      out.precision(2); // a.bc E xx
-      //out.setf(ios::scientific);
-      out.setf(ios::fixed);
-      for (int m = 0; m < exponent-1; m++) sev /= 10.0;
-      out << sev << " " << name << " ";
-      *(out.str()+out.pcount()) = 0;
-      return out.str();
+    if (!isInteger) {
+      if (max_sev >= 100.0) {
+	out.precision(2); // a.bc E xx
+	//out.setf(ios::scientific);
+	out.setf(ios::fixed);
+	for (int m = 0; m < exponent-1; m++) sev /= 10.0;
+	out << sev << " " << name << " ";
+	*(out.str()+out.pcount()) = 0;
+	return out.str();
+      } else if (max_sev < 0.1) {
+	out.precision(2); // a.bc E xx
+	out.setf(ios::fixed); // omit the E xx part 
+	exponent = get_exponent2(max_sev);
+	for (int m = 0; m < exponent; m++) sev *= 10.0;
+	out << sev << " " << p->get_name() << " ";
+	*(out.str()+out.pcount()) = 0;
+	return out.str();
+      }
     }
     if (isInteger && max_sev >= 1000) {
       out.clear();
@@ -1193,14 +1297,24 @@ string Cube::gen_func_children_label(int met_id, double sev, int view,
   if (uom.compare("occ") == 0) isInteger = true;
   string name = "subregions";
   if (view == 0) {// only if absolute view
-    if (!isInteger && max_sev >= 100.0) {
-      out.precision(2); // a.bc E xx
-      //out.setf(ios::scientific);
-      out.setf(ios::fixed);
-      for (int m = 0; m < exponent-1; m++) sev /= 10.0;
-      out << sev << " " << name << " ";
-      *(out.str()+out.pcount()) = 0;
-      return out.str();
+    if (!isInteger) {
+      if (max_sev >= 100.0) {
+	out.precision(2); // a.bc E xx
+	//out.setf(ios::scientific);
+	out.setf(ios::fixed);
+	for (int m = 0; m < exponent-1; m++) sev /= 10.0;
+	out << sev << " " << name << " ";
+	*(out.str()+out.pcount()) = 0;
+	return out.str();
+      } else if (max_sev < 0.1) {
+	out.precision(2); // a.bc E xx
+	out.setf(ios::fixed); // omit the E xx part 
+	exponent = get_exponent2(max_sev);
+	for (int m = 0; m < exponent; m++) sev *= 10.0;
+	out << sev << " " << p->get_name() << " ";
+	*(out.str()+out.pcount()) = 0;
+	return out.str();
+      }
     }
     if (isInteger && max_sev >= 1000) {
       out.precision(2); // a.bc E xx
@@ -1249,13 +1363,23 @@ string Cube::gen_loc_label(int met_id, int loc_id,
   if (uom.compare("occ") == 0) isInteger = true;
   string name = loc->get_name();
   if (view == 0) {// only if absolute view
-    if (!isInteger && max_sev >= 100.0) {
-      out.precision(2); // a.bc E xx
-      out.setf(ios::fixed);
-      for (int m = 0; m < exponent-1; m++) sev /= 10.0;   
-      out << sev << " " << name << " ";
-      *(out.str()+out.pcount()) = 0;
-      return out.str();
+    if (!isInteger) {
+      if (max_sev >= 100.0) {
+	out.precision(2); // a.bc E xx
+	out.setf(ios::fixed);
+	for (int m = 0; m < exponent-1; m++) sev /= 10.0;   
+	out << sev << " " << name << " ";
+	*(out.str()+out.pcount()) = 0;
+	return out.str();
+      } else if (max_sev < 0.1) {
+	out.precision(2); // a.bc E xx
+	out.setf(ios::fixed); // omit the E xx part 
+	exponent = get_exponent2(max_sev);
+	for (int m = 0; m < exponent; m++) sev *= 10.0;
+	out << sev << " " << p->get_name() << " ";
+	*(out.str()+out.pcount()) = 0;
+	return out.str();
+      }
     }
     if (isInteger && max_sev >= 1000) {
       out.precision(2); // a.bc E xx
