@@ -151,7 +151,7 @@ string CnodeTree::get_line(string path, string fname, int line) {
 
   path += "/";
   path += fname;
-  cout << "module name: " << path << endl;
+  // cout << "module name: " << path << endl;
   ifstream in (path.c_str());
   string ret;
   if (!in) {
@@ -203,6 +203,11 @@ void CnodeTree::display_region_code(int begn_ln, int end_ln) {
   in.close();
   
   frame->get_source_dlg()->show_module(successpath);
+  if (end_ln <= 0) {
+    end_ln = begn_ln;
+    cout << "Warning: try to display a callee with end_line <= 0!" << endl;
+    cout << "Only the begin_line is displayed." << endl;
+  }
   frame->get_source_dlg()->highlight(begn_ln, end_ln);
   frame->get_source_dlg()->show_line(begn_ln);
   frame->get_source_dlg()->Show(true);
@@ -283,12 +288,20 @@ void CnodeTree::OnSourceCode(wxCommandEvent& event) {
   frame->get_source_dlg()->set_module(mname);
   frame->get_source_dlg()->set_line(line);
   if (line <= 0) {
-    // user didn't define the line number, show the caller region
-    Region* caller = (Region*) csite->get_caller();
+    // If user doesn't define the line number, CUBE shows the callee region
+    // modified before LCI, Austin.
+    Region* callee = (Region*) csite->get_callee();
+    if (callee == NULL) {
+      cout << "WARNING: Action cancelled! trying to show an empty callee." << endl;
+      return;
+    }
+    mname = ((Module*)callee->get_mod())->get_name();
+    frame->get_source_dlg()->set_module(mname); // set to callee's module name
     string ln = get_line
-      (get_path(mname), get_base(mname), caller->get_begn_ln());
+      (get_path(mname), get_base(mname), callee->get_begn_ln());
     if (ln.empty()) return; // user click "Cancel"
-    display_region_code(caller->get_begn_ln(), caller->get_end_ln());
+    //cout << "region: " << callee->get_begn_ln() << ":" << callee->get_end_ln() << endl; 
+    display_region_code(callee->get_begn_ln(), callee->get_end_ln());
     return;
   }
   // if having a line number, always assume the line is correct.
@@ -315,7 +328,7 @@ CnodeTree::CnodeTree(wxWindow* parent) : Tree(parent) {
 }
 
 void CnodeTree::OnExpanded(wxTreeEvent& event) {
-  int id = (int) event.GetItem();
+  wxTreeItemId id = event.GetItem();
   PresenterFrame* frame = ((PresenterApp*)wxTheApp)->get_frame();
   if (frame->is_calltree()) {
     // refresh the callpath tree
@@ -347,7 +360,7 @@ void CnodeTree::OnExpanded(wxTreeEvent& event) {
 }
 
 void CnodeTree::OnCollapsed(wxTreeEvent& event) {
-  int id = (int) event.GetItem();
+  wxTreeItemId id = event.GetItem();
   PresenterFrame* frame = ((PresenterApp*)wxTheApp)->get_frame();
   if (frame->is_calltree()) {
     // refresh the callpath tree
@@ -379,7 +392,7 @@ void CnodeTree::OnCollapsed(wxTreeEvent& event) {
 }
 
 
-void CnodeTree::refresh_loc_tree4func(int id) {
+void CnodeTree::refresh_loc_tree4func(wxTreeItemId id) {
   PresenterFrame* frame = ((PresenterApp*)wxTheApp)->get_frame();
   int rid, rmode;
   // refresh the function tree
@@ -403,8 +416,7 @@ void CnodeTree::refresh_loc_tree4func(int id) {
 
 void CnodeTree::OnSelChanged(wxTreeEvent& event) {
   //int id = (int) event.GetItem();
-  int id = GetSelection(); // newly selected node.
-  int rid, rmode;
+  wxTreeItemId id = GetSelection(); // newly selected node.
   PresenterFrame* frame = ((PresenterApp*)wxTheApp)->get_frame();
   if (id == old_sel) return;
 
@@ -428,7 +440,7 @@ void CnodeTree::OnRightClick(wxTreeEvent& event) {
 }
 
 void CnodeTree::refresh (int met_id, int met_mode) {
-  cout << "call tree is updated:" << met_id << " " << met_mode << endl;
+  // cout << "call tree is updated:" << met_id << " " << met_mode << endl;
   target_met_id   = met_id;
   target_met_mode = met_mode;
   PresenterFrame* frame = ((PresenterApp*)wxTheApp)->get_frame();
@@ -438,14 +450,14 @@ void CnodeTree::refresh (int met_id, int met_mode) {
     // Refresh the callpath tree
     traverse_calltree(wxTreeCtrl::GetRootItem(), cube);
     // what's the currently selected cnode
-    int selection = GetSelection();
+    wxTreeItemId selection = GetSelection();
     int cnode_id = GetItemData(selection)->get_id();
     int cnode_mode = IsExpanded(selection) ? CUBE_EXCL : CUBE_INCL;
     frame->get_loc_tree()->refresh(met_id, met_mode, cnode_id, cnode_mode);
   } else {
     // Refresh the function tree
     traverse_functree(GetVRootItem(), cube);
-    int selection = GetSelection();
+    wxTreeItemId selection = GetSelection();
     refresh_loc_tree4func(selection);
   }
 }
