@@ -53,40 +53,38 @@ END_EVENT_TABLE()
 /* create and initialize a metric tree */
 void  PresenterFrame::build_met_tree() {
   if (cube == NULL || met_tree == NULL) return;
-  int firstroot = 0;
   vector<Metric*> roots = cube->get_met_roots();
   for (int k = 0; k < roots.size(); k++) {
     // several metric roots possibly
-    int rootId = met_tree->AddRoot("NULL", 101); // temporary label
+    wxTreeItemId rootId = met_tree->AddRoot("NULL", 101); // temporary label
     Metric* p = roots[k];
     met_tree->SetItemData(rootId, new NodeData(p->get_id()));
     ostrstream out;
-    met_tree->SelectItem(rootId);
-    if (firstroot == 0) {
-      firstroot = rootId;
+    if (k == 0) {
+      met_tree->SelectItem(rootId);
     }
     int mode[] = {CUBE_INCL, -1, -1};
     double sev = cube->get_vsev(mode, p->get_id());
-    string label = cube->gen_met_label(p->get_id(), sev);
+    string label = cube->gen_met_label(p->get_id(), sev, view_type());
     met_tree->SetItemText(rootId, label, cube->index(sev));
     
     /* build a wxTree from a cube metric tree*/
     vector<Metric*>  fromv[2]; // store ids for only two levels
-    vector<int>        tov[2];
+    vector<wxTreeItemId>        tov[2];
     int depth = 0;
     fromv[depth].push_back(p);       // add root of cube tree
     tov[depth].push_back(rootId);    // add root of wxWindows tree
-    while(fromv[depth].size() > 0) {
+    while(fromv[depth%2].size() > 0) {
       //building tree from level #depth to level #(depth+1) 
       for (int i = 0; i < fromv[depth%2].size(); i++) {
 	p = fromv[depth%2][i];
-	int pid = tov[depth%2][i];
+	wxTreeItemId pid = tov[depth%2][i];
 	for (int j = 0; j < p->num_children(); j++) {
 	  Metric* child = (Metric*) p->get_child(j);
 	  // convert to label
 	  ostrstream out2;
 	  sev = cube->get_vsev(mode, child->get_id());
-	  string label = cube->gen_met_label(child->get_id(), sev);
+	  string label = cube->gen_met_label(child->get_id(), sev, view_type());
 	  // store to the alternative vector
 	  fromv[(depth+1)%2].push_back(child);
 	  tov[(depth+1)%2].
@@ -100,13 +98,13 @@ void  PresenterFrame::build_met_tree() {
       depth++;
     } // end of construction of one tree
   } // end of construction of one forest
-  met_tree->SelectItem(firstroot);
+ 
 }
 
 /* create and initialize a cnode tree */
 void  PresenterFrame::build_cnode_tree() {
   if (cube == NULL || cnode_tree == NULL || !is_calltree()) return;
-  int rootId = cnode_tree->GetVRootItem();
+  wxTreeItemId rootId = cnode_tree->GetVRootItem();
   cnode_tree->DeleteChildren(rootId);
   if (cube->num_cnodes() == 0) return;
   vector<Cnode*> roots = cube->get_cnode_roots();
@@ -115,7 +113,7 @@ void  PresenterFrame::build_cnode_tree() {
     rootId = cnode_tree->AddRoot("NULL", 101); // temporary label
     Cnode* cnode = roots[k]; // get a root node
     cnode_tree->SetItemData(rootId, new NodeData(cnode->get_id()));
-    cnode_tree->SelectItem(rootId);
+    if (k == 0) cnode_tree->SelectItem(rootId);
     int sel_met = met_tree->GetItemData(met_tree->GetSelection())->get_id();
     // ostrstream out;
     int mode[] = {CUBE_INCL, CUBE_INCL, -1};
@@ -123,7 +121,7 @@ void  PresenterFrame::build_cnode_tree() {
     string label = cube->gen_cnode_label(sel_met, cnode->get_id(), sev);
     cnode_tree->SetItemText(rootId, label, cube->index(sev));
     vector<Cnode*>  fromv[2];        // store ids for only two levels
-    vector<int>        tov[2];
+    vector<wxTreeItemId>        tov[2];
     int depth = 0;
     fromv[depth].push_back(cnode);   // add root of cube tree
     tov[depth].push_back(rootId);    // add root of wxWindows tree
@@ -131,7 +129,7 @@ void  PresenterFrame::build_cnode_tree() {
       //building tree from depth to depth+1
       for (int i = 0; i < fromv[depth%2].size(); i++) {
 	cnode = fromv[depth%2][i];
-	int pid = tov[depth%2][i];
+	wxTreeItemId pid = tov[depth%2][i];
 	for (int j = 0; j < cnode->num_children(); j++) {
 	  Cnode* child = (Cnode*) cnode->get_child(j);
 	  // convert to label
@@ -151,7 +149,7 @@ void  PresenterFrame::build_cnode_tree() {
   }
 
   // if creation is not the first time, refresh it based on metric node info.
-  int id = met_tree->GetSelection(); // newly selected node.
+  wxTreeItemId id = met_tree->GetSelection(); // newly selected node.
   int met_id = met_tree->GetItemData(id)->get_id();
   int met_mode;
   if (met_tree->IsExpanded(id)) 
@@ -166,7 +164,7 @@ void  PresenterFrame::build_cnode_tree() {
 void PresenterFrame::build_func_tree() {
   wxTreeItemId sel_func; // default one in the beginning
   if (cube == NULL || cnode_tree == NULL || is_calltree()) return;
-  int rootId = cnode_tree->GetVRootItem();
+  wxTreeItemId rootId = cnode_tree->GetVRootItem();
   cnode_tree->DeleteChildren(rootId);
   vector<Module*> modv = get_cube()->get_mod_roots();
   if (modv.size() == 0) return; // no modules loaded ever
@@ -176,7 +174,7 @@ void PresenterFrame::build_func_tree() {
   for (int i = 0; i < modv.size(); i++) {
     Module* m = modv[i];
     int modid = m->get_id();
-    int pid = cnode_tree->AddRoot(m->get_name(), 0);
+    wxTreeItemId pid = cnode_tree->AddRoot(m->get_name(), 0);
     cnode_tree->SetItemData(pid, new NodeData(modid));
     // for each module, append its functions as child nodes
     for (int j = 0; j < m->num_region(); j++) {
@@ -185,7 +183,7 @@ void PresenterFrame::build_func_tree() {
       int mode[] = {CUBE_INCL, CUBE_INCL, -1};
       double sev = cube->get_vsev(mode, sel_met, r->get_id(), -1, false);
       string label=cube->gen_function_label(sel_met, r->get_id(), sev);
-      int funcid = 
+      wxTreeItemId funcid = 
 	cnode_tree->AppendItem(pid, label, cube->index(sev), 
 			       new NodeData(r->get_id()));
       if (i == 0 && j == 0) sel_func = funcid;
@@ -203,7 +201,7 @@ void PresenterFrame::build_func_tree() {
 /* create and initialize a location tree */
 void  PresenterFrame::build_loc_tree() {
   if (cube == NULL || loc_tree == NULL) return;
-  int rootId = loc_tree->AddRoot("NULL", 101);
+  wxTreeItemId rootId = loc_tree->AddRoot("NULL", 101);
   loc_tree->SelectItem(rootId);
   Location* p = (Location*) cube->get_loc(0);
   int process_level; // >= 0
@@ -225,7 +223,7 @@ void  PresenterFrame::build_loc_tree() {
     gen_loc_label(0, p->get_locid(), sev);
   loc_tree->SetItemText(rootId, label, cube->index(sev));
   vector<Location*>  fromv[2]; // store ids for only two levels
-  vector<int>        tov[2];
+  vector<wxTreeItemId>        tov[2];
   int depth = 0;
   fromv[depth].push_back(p);       // add root of cube tree
   tov[depth].push_back(rootId);    // add root of wxWindows tree
@@ -235,7 +233,7 @@ void  PresenterFrame::build_loc_tree() {
       if (depth == process_level && cube->get_max_thrdnum() == 1) 
 	continue; // no threads displayed
       p = fromv[depth%2][i];
-      int pid = tov[depth%2][i];
+      wxTreeItemId pid = tov[depth%2][i];
       for (int j = 0; j < p->num_children(); j++) {
 	Location* child = (Location*) p->get_child(j);
 	sev = cube->get_vsev(mode, 0, 0, child->get_locid());
@@ -362,7 +360,7 @@ void PresenterFrame::OnOpenFile(wxCommandEvent& event) {
 }
 /* Close current file */
 void PresenterFrame::OnCloseFile(wxCommandEvent& event) {
-  int rootId;
+  wxTreeItemId rootId;
   rootId = met_tree->GetVRootItem();
   met_tree->DeleteChildren(rootId);
   rootId = cnode_tree->GetVRootItem();
@@ -390,7 +388,7 @@ void PresenterFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
   sizer->Add(icon, 0, wxCENTER|wxLEFT, 30);
   sizer->Add(new wxStaticText
 	     (about, -1, 
-	      "This is the CUBE Version 1.0.\n(C) ICL at UTK 2004", 
+	      "This is the CUBE Version 1.0.5.\n(C) ICL at UTK 2004", 
 	      wxDefaultPosition, wxSize(280, -1)),
 	     0,
 	     wxCENTER|wxLEFT, 10);
